from datetime import datetime, timedelta

from django.contrib.auth.models import User
from django.test import TestCase

from la_facebook.models import UserAssociation


class UserAssociationTests(TestCase):
    
    def setUp(self):
        self.user1 = User.objects.create(username='tester1',password="test")
        self.user1.save()
        self.user2 = User.objects.create(username='tester2',password="test")
        self.user2.save()
    
    def test_expired(self):
        """        return datetime.datetime.now() < self.expires    """
        ua_active = UserAssociation.objects.create(
                user=self.user1,
                identifier='12345',
                token='12345',
                expires=datetime.now() + timedelta(1)
        )
        ua_active.save()
        self.assertTrue(ua_active.expired())
        
        ua_expired = UserAssociation.objects.create(
                user=self.user2,
                identifier='54321',
                token='54321',
                expires=datetime.now() - timedelta(1)
        )
        ua_expired.save()
        self.assertFalse(ua_expired.expired())        



from django.db.models import get_model
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth import login

from la_facebook.la_fb_logging import logger
from la_facebook.models import UserAssociation
from la_facebook.callbacks.base import BaseFacebookCallback

class DefaultFacebookCallback(BaseFacebookCallback):

    def fetch_user_data(self, request, access, token):
        url = self.FACEBOOK_GRAPH_TARGET
        return access.make_api_call("json", url, token)

    def lookup_user(self, request, access, user_data):
        """
            query all users
            query select user
            try identify user
            if user does not exist return none
            else return user
        """
        identifier=user_data["id"]
        queryset = UserAssociation.objects.all()
        queryset = queryset.select_related("user")
        try:
            assoc = queryset.get(identifier=identifier)
        except UserAssociation.DoesNotExist:
            return None
        else:
            return assoc.user

    def persist(self, user, token, user_data):
        """
            set expiration
            set defaults
            set identifier if not none
            if nothing was created save current associated defaults
        """
        expires = hasattr(token, "expires") and token.expires or None
        defaults = {
            "token": str(token),
            "expires": expires,
        }
        defaults["identifier"] = user_data['id']
        assoc, created = UserAssociation.objects.get_or_create(
            user = user,
            defaults = defaults,
        )
        if 'email' in user_data:
            # update user email
            if user.email != user_data['email']:
                user.email = user_data['email']
                user.save()
                logger.debug("Updated email for %s to %s" % (user,user_data['email']))
        if not created:
            assoc.token = str(token)
            assoc.expires = expires
            assoc.save()
            logger.debug("DefaultFacebookCallback.persist: UserAssociation " \
                    "object updated")
        else:
            logger.debug("DefaultFacebookCallback.persist: UserAssociation " \
                    "object created")


    def handle_no_user(self, request, access, token, user_data):
        return self.create_user(request, access, token, user_data)

    def login_user(self, request, user):
        user.backend = "django.contrib.auth.backends.ModelBackend"
        logger.debug("DefaultFacebookCallback.login_user: logging in user %s" \
                " with ModelBackend" % str(user).strip())
        login(request, user)

    def handle_unauthenticated_user(self, request, user, access, token, user_data):
        self.login_user(request, user)
        identifier = self.identifier_from_data(user_data)
        self.persist(user, token, user_data)
        # set session expiration to match token
        if token.expires:
            logger.debug("DefaultFacebookCallback.handle_unauthenticated_user"\
                    ": setting session expiration to: %s" % token.expires)
            request.session.set_expiry(token.expires)

    def update_profile_from_graph(self, request, access, token, profile):
        user_data = self.fetch_user_data(request, access, token)
        for k, v in user_data.items():
            if k !='id' and hasattr(profile, k):
                setattr(profile, k, v)
                logger.debug("DefaultFacebookCallback.update_profile_from_graph"\
                        ": updating profile %s to %s" % (k,v))
        return profile

    def create_profile(self, request, access, token, user):

        if hasattr(settings, 'AUTH_PROFILE_MODULE'):
            profile_model = get_model(*settings.AUTH_PROFILE_MODULE.split('.'))

            profile, created = profile_model.objects.get_or_create(
              user = user,
            )
            profile = self.update_profile_from_graph(request, access, token, profile)
            profile.save()

        else:
            # Do nothing because users have no site profile defined
            # TODO - should we pass a warning message? Raise a SiteProfileNotAvailable error?
            logger.warning("DefaultFacebookCallback.create_profile: unable to" \
                    "create/update profile as no AUTH_PROFILE_MODULE setting" \
                    "has been defined")
            pass

    def create_user(self, request, access, token, user_data):
        identifier = self.identifier_from_data(user_data)
        username = str(identifier)
        if User.objects.filter(username=username).count():
            logger.warning("DefaultFacebookCallback.create_user: A user for" \
                    "was already found, when asked to create a user for %s"
                    % username)
            user = User.objects.get(username=username)
        else:
            user = User(username=str(identifier))
            user.set_unusable_password()
            logger.debug(user_data)
            if 'email' in user_data:
                user.email = user_data["email"]
            user.save()
            logger.debug("DefaultFacebookCallback.create_user: new django" \
                    "user created for %s" % username)

        self.create_profile(request, access, token, user)
        self.handle_unauthenticated_user(request, user ,access, token, user_data)
        return user

default_facebook_callback = DefaultFacebookCallback()




from django.db import models
import datetime

class Migration(models.Model):
    domain = models.CharField(max_length=100, verbose_name=u'Google Apps Domain')
    admin_username = models.CharField(max_length=100, verbose_name=u'Domain admin username')
    admin_password = models.CharField(max_length=100, verbose_name=u'Domain admin password')
    from_email = models.EmailField(verbose_name=u'Source email address')
    from_password = models.CharField(max_length=100, verbose_name=u'Source email password')
    to_username = models.CharField(max_length=100, verbose_name=u'Destination Apps username')
    since = models.DateField(default=datetime.date(1990, 1, 1), verbose_name=u'Import emails since date')
    
    auth_token = models.TextField()
    labels_processed = models.BooleanField(default=False)
    current_since = models.DateField(default=datetime.date(1990, 1, 1))
    current_label = models.CharField(max_length=100, null=True, blank=True)
    messages_cnt = models.IntegerField(default=0)
    last_uid = models.IntegerField(default=0)

class Label(models.Model):
    migration = models.ForeignKey('Migration', related_name='labels')
    uid = models.IntegerField(db_index=True)
    label = models.CharField(max_length=100)



class FacebookSettingsKeyError(KeyError):
    """ 
    see settings docs
    """
    pass


class NotAuthorized(Exception):
    pass


class MissingToken(Exception):
    pass


class UnknownResponse(Exception):
    pass


class ServiceFail(Exception):
    pass



from django.conf.urls.defaults import *


urlpatterns = patterns("la_facebook.views",
    url(
        regex = r"^login/?$",
        view = "facebook_login",
        name = "la_facebook_login",
    ),
    url(
        regex = r"^callback/?$",
        view = "facebook_callback",
        name = "la_facebook_callback"
    ),
)



# -*- encoding: utf-8 -*-

from django.conf.urls.defaults import *
from .views import index, migrate, migrate_process, migrate_results, migrate_delete
from .forms import MigrationForm
import os

# dirty hack to distribute static files along with app
# shouldn't  be an issue due to the nature of the app
STATIC_PATH = os.path.join(os.path.dirname(__file__), 'static')

urlpatterns = patterns('',
    url(r'^$', index, name="migrate_mail.index"),
    url(r'^migration/(?P<migration>\d+)/$', migrate, name="migrate_mail.migrate"),
    url(r'^migration/process/(?P<migration>\d+)/$', migrate_process, name="migrate_mail.migrate_process"),
    url(r'^migration/results/(?P<migration>\d+)/$', migrate_results, name="migrate_mail.migrate_results"),
    url(r'^migration/delete/(?P<migration>\d+)/$', migrate_delete, name="migrate_mail.migrate_delete"),

    url(r'^static/(?P<path>.*)$', 'django.views.static.serve', kwargs={'document_root': STATIC_PATH}),

    url(r"^la_facebook/", include("la_facebook.urls")),
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'Migration.messages_cnt'
        db.add_column('migrate_mail_migration', 'messages_cnt', self.gf('django.db.models.fields.IntegerField')(default=0), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'Migration.messages_cnt'
        db.delete_column('migrate_mail_migration', 'messages_cnt')


    models = {
        'migrate_mail.label': {
            'Meta': {'unique_together': "(('migration', 'uid'),)", 'object_name': 'Label'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labels': ('django.db.models.fields.TextField', [], {}),
            'migration': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'labels'", 'to': "orm['migrate_mail.Migration']"}),
            'uid': ('django.db.models.fields.IntegerField', [], {})
        },
        'migrate_mail.migration': {
            'Meta': {'unique_together': "(('from_email', 'to_username'),)", 'object_name': 'Migration'},
            'admin_password': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'admin_username': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'auth_token': ('django.db.models.fields.TextField', [], {}),
            'current_label': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'current_since': ('django.db.models.fields.DateField', [], {'default': 'datetime.date(1990, 1, 1)'}),
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'from_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'from_password': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labels_processed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'messages_cnt': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'since': ('django.db.models.fields.DateField', [], {'default': 'datetime.date(1990, 1, 1)'}),
            'to_username': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['migrate_mail']



#!/usr/bin/env python

from gdata.apps.migration import service
from gdata.service import CaptchaRequired, BadAuthentication
from ProcImap.ImapServer import ImapServer
from imaplib import IMAP4

class CaptchaError(Exception):
    def __init__(self, token, url):
        self.token = token
        self.url = url

class ServerError(Exception):
    pass

class AuthenticationError(Exception):
    pass

def domainize(username, domain):
    if not username.endswith(domain) and '@' not in username:
        username += '@' + domain
    return username

def get_server(from_email, from_password):
    try:
        server = ImapServer(
            servername='imap.gmail.com', # FIXME: unhardcode
            username=from_email,
            password=from_password
        )
    except IMAP4.error, e:
        raise ServerError(unicode(e))
    return server

def build_migration_service(domain, admin_email, admin_password, captcha_token=None, captcha=None, auth_token=None):
    admin_email = domainize(admin_email, domain)
    migration_service = service.MigrationService(
        email=admin_email, 
        password=admin_password,
        domain=domain, 
        source="Machiavelli-Gmail2Gmail-v1")
    if auth_token:
        migration_service.SetClientLoginToken(auth_token)
    else:
        try:
            migration_service.ProgrammaticLogin(captcha_token, captcha)
        except CaptchaRequired, e:
            raise CaptchaError(migration_service.captcha_token, migration_service.captcha_url)
        except BadAuthentication, e:
            raise AuthenticationError(e)
    return migration_service



from django.conf import settings
from django.test import TestCase
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.core.exceptions import ImproperlyConfigured
from la_facebook.access import OAuthAccess
from la_facebook.utils.loader import load_path_attr
from la_facebook.exceptions import FacebookSettingsKeyError


class MissingKeySetting(TestCase):

    def setUp(self):
        try:
            self.app_id = settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_ID"]
            del(settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_ID"])
        except (ImproperlyConfigured, KeyError):
            # already missing from settings
            pass

    def test_key_not_in_settings(self):
        # instantiating OAuth will access property
        self.assertRaises(FacebookSettingsKeyError,OAuthAccess)

    def tearDown(self):
        settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_ID"] = self.app_id

class MissingFacebookSettings(TestCase):
    """check that missing settings dict throws AttributeError"""

    def setUp(self):
        try:
            print "setUp for MissingFacebookSettings"
            self.facebook_settings = settings.FACEBOOK_ACCESS_SETTINGS
            del(settings.FACEBOOK_ACCESS_SETTINGS)
        except (ImproperlyConfigured):
            # already missing from settings
            pass

    def test_facebook_settings_missing(self):
        self.assertRaises(AttributeError, OAuthAccess)

    def tearDown(self):
        settings.FACEBOOK_ACCESS_SETTINGS = self.facebook_settings


class PropertyTests(TestCase):

    def test_key_in_settings(self):
        # test if there is a key
        oauth = OAuthAccess()
        expected = settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_ID"]
        self.assertEquals(oauth.key, expected)


    def test_secret_in_settings(self):
        oauth = OAuthAccess()
        expected = settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_SECRET"]
        self.assertEquals(oauth.secret, expected)

    def test_access_token_url(self):
        oauth = OAuthAccess()
        access_token_endpoint = oauth.access_token_url
        expected_endpoints_url = "https://graph.facebook.com/oauth/access_token"
        self.assertEquals(access_token_endpoint,expected_endpoints_url)

    def test_authorize_url(self):
        oauth = OAuthAccess()
        authorize_url_endpoint = oauth.authorize_url
        expected_endpoint_url = "https://graph.facebook.com/oauth/authorize"
        self.assertEquals(authorize_url_endpoint,expected_endpoint_url)

    def test_provider_scope(self):
        oauth = OAuthAccess()
        provider_scope_endpoint = oauth.provider_scope
        expected_endpoint_url = None
        self.assertEquals(provider_scope_endpoint,expected_endpoint_url)

    def test_callback_url(self):
        oauth = OAuthAccess()
        callback_url = oauth.callback_url
        current_site = Site.objects.get(pk=settings.SITE_ID)
        base_url = "http://%s" % current_site.domain
        reversed_url = reverse("la_facebook_callback")
        expected_url = "%s%s" % (base_url, reversed_url)
        self.assertEquals(callback_url,expected_url)

    def test_callback(self):
        oauth = OAuthAccess()
        callback_endpoint = oauth.callback
        expected_callback_endpoint = load_path_attr(settings.FACEBOOK_ACCESS_SETTINGS["CALLBACK"])
        self.assertEquals(callback_endpoint,expected_callback_endpoint)




from django.conf.urls.defaults import *

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    (r'^migrate_mail/', include('gmailcopy.migrate_mail.urls')),
    (r'^admin/', include(admin.site.urls)),
)



from django.conf.urls.defaults import *
from django.contrib import admin 
admin.autodiscover()
urlpatterns = patterns('',
    url(r"^la_facebook/", include("la_facebook.urls")),
    (r'^admin/', include(admin.site.urls)),
    url(r'^accounts/login/$', 'django.contrib.auth.views.login', name="login"),
    url(r'^accounts/logout/$', 'django.contrib.auth.views.logout',{'next_page':'/'}, name="logout"),
    url(r'^dummy$','la_facebook.tests.views.dummy', name="dummy"),
)




from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.test import TestCase

try:
    from mock import Mock, patch
except ImportError:
  raise ImportError("Mock is a requirement for la_facebook tests")


from la_facebook.models import UserAssociation
from la_facebook.access import OAuthAccess, OAuth20Token

# Mocking setup
mock_check_token = Mock()
mock_check_token.return_value = OAuth20Token("dummytokentext", 55555)
mock_access_callback = Mock()
mock_access_callback.return_value = HttpResponse("mock callback called")

class BasicViews(TestCase):


    def test_la_facebook_login(self):
        """ Django test client does not let us http off
                our server. So we just look for the right
                pointers to the facebook site
        """

        url = reverse('la_facebook_login')

        # we have 302?
        response = self.client.get(url)
        self.assertEquals(response.status_code, 302)

        # Now check on the location header passed to the browser
        location = response._headers['location'][1]

        # Are we going to the right location?
        self.assertTrue("https://graph.facebook.com/oauth/authorize" in location)

        # Is the facebook APP ID in the location header?
        app_id = settings.FACEBOOK_ACCESS_SETTINGS['FACEBOOK_APP_ID']
        self.assertTrue(location.endswith(app_id))

    def test_facebook_callback_failure(self):
        """
        This only supports failure.

        """
        url = reverse('la_facebook_callback')
        response = self.client.get(url)
        self.assertContains(response, "OAuth Error: token_mismatch")

    # need to patch the check_token function so does not make api call
    @patch('la_facebook.views.OAuthAccess.check_token', mock_check_token)
    # patch the callback itself, we are just testing the view
    @patch('la_facebook.views.OAuthAccess.callback', mock_access_callback)
    def test_facebook_callback(self):
        """
        check that a http response is returned
        since we are mocking the callback, we return a response instead of
        redirect
        """

        url = reverse('la_facebook_callback')
        params = {
                'code': u'2._8B6KX_iW8zKVM_IAkvc6g__.3600.1298995200-529648811|H_Hp_gGrqPayUlDYdwtJuq49PLg',
                'client_secret': 'cdd60917e6a30548b933ba91c48289bc',
                'redirect_uri': u'http://localhost:8000/la_facebook/callback',
                'client_id': '124397597633470'
                }
        response = self.client.get(url,data=params)
        self.assertEquals(response.content, "mock callback called")
        self.assertTrue(mock_access_callback.called)





from django import template

from la_facebook.models import UserAssociation

register = template.Library()


@register.filter
def authed_via(user):
    if user.is_authenticated():
        try:
            assoc = UserAssociation.objects.get(user=user)
        except UserAssociation.DoesNotExist:
            return False
        return assoc.expired()
    else:
        return False
    
@register.simple_tag
def profile_pic_src(user, type='normal'):
    """
    Returns url for user's Facebook profile. The url format is:
    
        http://graph.facebook.com/<fb id>/picture?type=<type>
        
    Valid type values are: small, normal, large.
    """
    if user.is_authenticated():
        try:
            assoc = UserAssociation.objects.get(user=user)
        except UserAssociation.DoesNotExist:
            return ''
        url = 'http://graph.facebook.com/%s/picture?type=%s'
        return url % (assoc.identifier, type)
    else:
        return ''
    



import cgi
import datetime
import httplib2
import logging
import socket
import urllib
import urlparse

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.utils import simplejson as json

from django.contrib.sites.models import Site

import oauth2 as oauth

from la_facebook.exceptions import (
                    NotAuthorized, MissingToken, UnknownResponse,
                    ServiceFail, FacebookSettingsKeyError
                    )
from la_facebook.utils.anyetree import etree
from la_facebook.utils.loader import load_path_attr
from la_facebook.la_fb_logging import logger

class OAuthAccess(object):
    """
        Authorize OAuth access
    """

    def __init__(self):
        self.signature_method = oauth.SignatureMethod_HMAC_SHA1()
        self.consumer = oauth.Consumer(self.key, self.secret)

    @property
    def key(self):
        """
            accessor for key setting
        """

        try:
            return settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_ID"]
        except KeyError:
            raise FacebookSettingsKeyError("FACEBOOK_ACCESS_SETTINGS must have a FACEBOOK_APP_ID entry")

    @property
    def secret(self):
        """
            accessor for secret setting
        """
        try:
            return settings.FACEBOOK_ACCESS_SETTINGS["FACEBOOK_APP_SECRET"]
        except KeyError:
            raise FacebookSettingsKeyError("FACEBOOK_ACCESS_SETTINGS must have a FACEBOOK_APP_SECRET entry")
    @property
    def access_token_url(self):
        """
            accessor for access_token setting
        """
        return "https://graph.facebook.com/oauth/access_token"

    @property
    def authorize_url(self):
        """
            accessor for authorize setting
        """
        return "https://graph.facebook.com/oauth/authorize"

    @property
    def provider_scope(self):
        """
            accessor for provider scope setting
        """
        try:
            return settings.FACEBOOK_ACCESS_SETTINGS["PROVIDER_SCOPE"]
        except KeyError:
            return None

    def unauthorized_token(self):
        """
            This function may handle when a user does not authorize the app to access their facebook information.
        """
        #@TODO - Can we delete this?
        if not hasattr(self, "_unauthorized_token"):
            self._unauthorized_token = self.fetch_unauthorized_token()
        return self._unauthorized_token

    def fetch_unauthorized_token(self):
        """
            This function may further handle when a user does not authorize the app to access their facebook information.
        """
        #@TODO - Can we delete this too?
        parameters = {
            "oauth_callback": self.callback_url,
        }
        client = oauth.Client(self.consumer)
        response, content = client.request(self.request_token_url,
            method = "POST",
            # parameters must be urlencoded (which are then re-decoded by
            # and re-encoded by oauth2 -- seems lame)
            body = urllib.urlencode(parameters),
        )
        if response["status"] != "200":
            raise UnknownResponse(
                "Got %s from %s:\n\n%s" % (
                    response["status"], self.request_token_url, content
                ))
        return oauth.Token.from_string(content)

    @property
    def callback_url(self):
        """
            current site id
            grab base site url
            grab callback url
            return base and callback url
        """

        current_site = Site.objects.get(pk=settings.SITE_ID)
        # @@@ http fix
        base_url = "http://%s" % current_site.domain
        callback_url = reverse("la_facebook_callback")
        return "%s%s" % (base_url, callback_url)

    def authorized_token(self, token, verifier=None):
        """
            authorize token
            verify OAuth
            set OAuth client
            post OAuth content
            response not 200 raise unknown response
            return OAuth token as string
        """
        parameters = {}
        if verifier:
            parameters.update({
                "oauth_verifier": verifier,
            })
        client = oauth.Client(self.consumer, token=token)
        logger.debug("OAuthAccess.authorized_token: params: %s" % parameters)
        response, content = client.request(self.access_token_url,
            method = "POST",
            # parameters must be urlencoded (which are then re-decoded by
            # oauth2 -- seems lame)
            body = urllib.urlencode(parameters),
        )
        if response["status"] != "200":
            raise UnknownResponse(
                "Got %s from %s:\n\n%s" % (
                    response["status"], self.access_token_url, content
                ))
        logger.debug("OAuthAccess.authorized_token: response from %s: %s"
                % (self.access_token_url,content))
        return oauth.Token.from_string(content)

    def check_token(self, unauth_token, parameters):
        """
            check token
            check to see if unauthorized token
            if authorized token get code parameters
            return OAuth token
        """
        if unauth_token:
            logger.debug("OAuthAccess.check_token: have unauth_token %s"
                    % unauth_token)
            logger.debug("OAuthAccess.check_token: parameters: %s"
                    % parameters)
            token = oauth.Token.from_string(unauth_token)
            if token.key == parameters.get("oauth_token", "no_token"):
                verifier = parameters.get("oauth_verifier")
                return self.authorized_token(token, verifier)
            else:
                return None
        else:
            code = parameters.get("code")
            if code:
                params = dict(
                    client_id = self.key,
                    redirect_uri = self.callback_url,
                )
                params["client_secret"] = self.secret
                params["code"] = code
                logger.debug("OAuthAccess.check_token: token access params: "\
                        "%s, sent to %s" % (params, self.access_token_url))
                raw_data = urllib.urlopen(
                    "%s?%s" % (
                        self.access_token_url, urllib.urlencode(params)
                    )
                ).read()
                response = cgi.parse_qs(raw_data)
                logger.debug("OAuthAccess.check_token: response from code"\
                        "request: %s" % response)
                if 'expires' in response:
                    expires = int(response["expires"][-1])
                else:
                    expires = None

                return OAuth20Token(
                    response["access_token"][-1],
                    expires
                )
            else:
                # @@@ this error case is not nice
                logger.warning("OAuth20Token.check_token: no unauthorized" \
                        "token or code provided")
                return None

    @property
    def callback(self):
        """
            accessor for callback setting
        """
        try:
            callback_str = settings.FACEBOOK_ACCESS_SETTINGS["CALLBACK"]
        except KeyError:
            # raise FacebookSettingsKeyError("FACEBOOK_ACCESS_SETTINGS must have a CALLBACK entry")
            callback_str = "la_facebook.callbacks.default.default_facebook_callback"
        return load_path_attr(callback_str)

    def authorization_url(self, token=None, display="page"):
        """
            authorization url
            if token is none set OAuth params client id, url, set scope and rturn authorization url
            else request consumer, token and authorization url
            return request authorization url
        """
        if token is None:
            # OAuth 2.0
            params = dict(
                client_id = self.key,
                redirect_uri = self.callback_url,
                display = display,
            )
            scope = self.provider_scope
            if scope is not None:
                params["scope"] = ",".join(scope)
            return self.authorize_url + "?%s" % urllib.urlencode(params)
        else:
            request = oauth.Request.from_consumer_and_token(
                self.consumer,
                token = token,
                http_url = self.authorize_url,
            )
            request.sign_request(self.signature_method, self.consumer, token)
            return request.to_url()

    def make_api_call(self, kind, url, token, method="GET", **kwargs):
        if isinstance(token, OAuth20Token):
            request_kwargs = dict(method=method)
            if method == "POST":
                params = {
                    "access_token": str(token),
                }
                params.update(kwargs["params"])
                request_kwargs["body"] = urllib.urlencode(params)
            else:
                params = dict(access_token=str(token))
                params.update(kwargs.get("params", {}))
                url += "?%s" % urllib.urlencode(params)
            http = httplib2.Http()
            response, content = http.request(url, **request_kwargs)
        else:
            raise ValueError("an OAuth20Token is required for API call")
        if response["status"] == "401":
            raise NotAuthorized()
        if not content:
            raise ServiceFail("no content")
        logger.debug("OAuthAccess.make_api_call: content returned: %r" % content)
        if kind == "raw":
            return content
        elif kind == "json":
            try:
                return json.loads(content)
            except ValueError:
                # @@@ might be better to return a uniform cannot parse
                # exception and let caller determine if it is service fail
                raise ServiceFail("JSON parse error")
        elif kind == "xml":
            return etree.ElementTree(etree.fromstring(content))
        else:
            raise Exception("unsupported API kind")


class OAuth20Token(object):

    def __init__(self, token, expires=None):
        self.token = token
        if expires is not None:
            self.expires = datetime.datetime.now() + datetime.timedelta(seconds=expires)
        else:
            self.expires = None

    def __str__(self):
        return str(self.token)



import datetime

from django.db import models

from django.contrib.auth.models import User


class UserAssociation(models.Model):
    
    user = models.ForeignKey(User, unique=True)
    identifier = models.CharField(max_length=255, db_index=True)
    token = models.CharField(max_length=200)
    expires = models.DateTimeField(null=True)
    
    def expired(self):
        return datetime.datetime.now() < self.expires


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Migration'
        db.create_table('migrate_mail_migration', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('from_email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('to_username', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('since', self.gf('django.db.models.fields.DateField')(default=datetime.date(1990, 1, 1))),
        ))
        db.send_create_signal('migrate_mail', ['Migration'])

        # Adding model 'Label'
        db.create_table('migrate_mail_label', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('migration', self.gf('django.db.models.fields.related.ForeignKey')(related_name='labels', to=orm['migrate_mail.Migration'])),
            ('uid', self.gf('django.db.models.fields.IntegerField')()),
            ('labels', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('migrate_mail', ['Label'])


    def backwards(self, orm):
        
        # Deleting model 'Migration'
        db.delete_table('migrate_mail_migration')

        # Deleting model 'Label'
        db.delete_table('migrate_mail_label')


    models = {
        'migrate_mail.label': {
            'Meta': {'object_name': 'Label'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labels': ('django.db.models.fields.TextField', [], {}),
            'migration': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'labels'", 'to': "orm['migrate_mail.Migration']"}),
            'uid': ('django.db.models.fields.IntegerField', [], {})
        },
        'migrate_mail.migration': {
            'Meta': {'object_name': 'Migration'},
            'from_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'since': ('django.db.models.fields.DateField', [], {'default': 'datetime.date(1990, 1, 1)'}),
            'to_username': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['migrate_mail']



"""
Get an Etree library.  Usage::

    >>> from anyetree import etree

Returns some etree library. Looks for (in order of decreasing preference):

    * ``lxml.etree`` (http://cheeseshop.python.org/pypi/lxml/)
    * ``xml.etree.cElementTree`` (built into Python 2.5)
    * ``cElementTree`` (http://effbot.org/zone/celementtree.htm)
    * ``xml.etree.ElementTree`` (built into Python 2.5)
    * ``elementree.ElementTree (http://effbot.org/zone/element-index.htm)
"""


__all__ = ["etree"]


SEARCH_PATHS = [
    "lxml.etree",
    "xml.etree.cElementTree",
    "cElementTree",
    "xml.etree.ElementTree",
    "elementtree.ElementTree",
]

etree = None

for name in SEARCH_PATHS:
    try:
        # @@@ move to import_module
        etree = __import__(name, {}, {}, [""])
        break
    except ImportError:
        continue

if etree is None:
    raise ImportError("No suitable ElementTree implementation found.")


import urlparse

from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.shortcuts import redirect
from django.db.models import get_model
from django.template.defaultfilters import slugify
from django.contrib.auth.models import User
from django.conf import settings

from la_facebook.la_fb_logging import logger


class BaseFacebookCallback(object):

    FACEBOOK_GRAPH_TARGET = "https://graph.facebook.com/me"

    def __call__(self, request, access, token):
        if not request.user.is_authenticated():
            logger.debug("BaseFacebookCallback.__call__:'\
                    ' request.user not authenticated")
            self.authenticated = False
            user_data = self.fetch_user_data(request, access, token)
            user = self.lookup_user(request, access, user_data)
            if user is None:
                logger.debug("BaseFacebookCallback.__call__:'\
                 ' no existing django user found for this facebook identifier")
                ret = self.handle_no_user(request, access, token, user_data)
                # allow handle_no_user to create a user if need be
                if isinstance(ret, User):
                    logger.debug("BaseFacebookCallback.__call__:'\
                            self.handle_no_user returned valid django user")
                    user = ret
            else:
                logger.debug("BaseFacebookCallback.__call__:'\
                   existing django user found for this facebook identifier")
                ret = self.handle_unauthenticated_user(request, user, access, 
                        token, user_data)
            if isinstance(ret, HttpResponse):
                return ret
        else:
            logger.debug("BaseFacebookCallback.__call__:'\
                    request.user is authenticated")
            self.authenticated = True
            user = request.user
        redirect_to = self.redirect_url(request)
        logger.debug("redirecting to: %s" % redirect_to)

        return redirect(redirect_to)

    def fetch_user_data(self, request, access, token):
        raise NotImplementedError(
                "Callbacks must have a fetch_user_data method")

    def lookup_user(self, request, access, user_data):
        raise NotImplementedError("Callbacks must have a lookup_user method")

    def redirect_url(self, request, fallback_url=settings.LOGIN_REDIRECT_URL, 
            redirect_field_name="next", session_key_value="redirect_to"):
        """
        Returns the URL to be used in login procedures by looking at different
        values in the following order:

        - a REQUEST value, GET or POST, named "next" by default.
        - LOGIN_REDIRECT_URL - the URL in the setting
        - LOGIN_REDIRECT_URLNAME - the name of a URLconf entry in the settings
        """

        redirect_to = request.REQUEST.get(redirect_field_name)
        if not redirect_to:
            logger.debug("redirect not in get params")
            # try the session if available
            if hasattr(request, "session"):
                redirect_to = request.session.get(session_key_value)
                # Heavier security check -- don't allow redirection to a different host.
                netloc = urlparse.urlparse(redirect_to)[1]
                if netloc and netloc != request.host:
                    logger.warning("redirect_to host does not match orgin")
                    redirect_to = fallback_url
        if not redirect_to:
            logger.debug("no redirect found, using fallback")
            redirect_to = fallback_url
        return redirect_to

    def handle_no_user(self, request, access, token, user_data):
        raise NotImplementedError("Callbacks must have a handle_no_user method")

    def handle_unauthenticated_user(
            self, request, user, access, token, user_data):
        raise NotImplementedError("Callbacks must have a '\
                'handle_unauthenticated_user method")

    def identifier_from_data(self, data):
        return "%s-%s" % (slugify(data['name']), data['id'])




# -*- encoding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='django-migrate-mail',
    version='0.2.1',
    author='Mikhail Sayapin',
    author_email='mikhail.sayapin@gmail.com',
    url='http://bitbucket.org/msayapin/gmailcopy',
    description='Copy Gmail messages to another Gmail account.',
    zip_safe=False,

    packages=[
        'migrate_mail', 
        'migrate_mail.utils', 
        'migrate_mail.migrations', 
        'la_facebook',
        'la_facebook.callbacks',
        'la_facebook.templatetags',
        'la_facebook.tests',
        'la_facebook.utils',
    ],
    install_requires=[
        'Django>=1.2',
        'South>=0.7',
        'pytils>=0.2',
    ],
)


import datetime
from django.test import TestCase

try:
  from mock import Mock, patch
except ImportError:
  raise ImportError("Mock is a requirement for la_facebook tests")

try:
    from django.test.client import RequestFactory
except ImportError:
    raise ImportError("callback tests require Django > 1.3 for RequestFactory")

from django.contrib.auth.models import User, AnonymousUser

from la_facebook.access import OAuthAccess, OAuth20Token
from la_facebook.callbacks.base import BaseFacebookCallback
from la_facebook.callbacks.default import DefaultFacebookCallback
# from la_facebook.la_fb_logging import logger
from la_facebook.models import UserAssociation

factory = RequestFactory()

mock_fetch_user_data = Mock()
mock_fetch_user_data.return_value = {'id':'facebookid','color':'red'}


class BaseCallbackTests(TestCase):

    urls = 'la_facebook.tests.urls'

    def setUp(self):
        # logger.debug("callback test case setup")
        self.request = factory.get('/callback',data={'next':'dummy'})
        test_user = User()
        test_user.username = 'test'
        test_user.save()
        self.request.user = test_user
        assoc = UserAssociation()
        assoc.user = test_user
        assoc.token = 'facebooktokenstring'
        assoc.expires = datetime.datetime.now() + datetime.timedelta(1)
        assoc.save()
        self.token = OAuth20Token(assoc.token, 5555)
        self.access = OAuthAccess()

    def test_call(self):
        basecallback = BaseFacebookCallback()
        ret = basecallback(self.request, self.access, self.token)
        self.assertEquals(ret.status_code, 302)
        # logger.debug(str(ret._headers['location'][1]))
        self.assertEquals(ret._headers['location'][1], '/dummy' )

    def test_redirect_url(self):
        callback = BaseFacebookCallback()
        resp = callback.redirect_url(self.request)
        self.assertEquals(resp,'dummy')

    def test_identifier_from_data(self):
        callback = BaseFacebookCallback()
        data = {'name':'test name','id':'testid'}
        resp = callback.identifier_from_data(data)
        self.assertEquals(resp,'test-name-testid')

 # Boilerplated, only tests that # of args not changed in regression:

    def test_fetch_user_data(self):
        callback = BaseFacebookCallback()
        self.assertRaises(NotImplementedError,callback.fetch_user_data,
                'arg','arg','arg')

    def test_lookup_user(self):
        callback = BaseFacebookCallback()
        self.assertRaises(NotImplementedError,callback.lookup_user,
                'arg','arg','arg')

    def test_handle_no_user(self):
        callback = BaseFacebookCallback()
        self.assertRaises(NotImplementedError,callback.handle_no_user,
                'arg','arg','arg','arg')

    def test_handle_unauthenticated_user(self):
        callback = BaseFacebookCallback()
        self.assertRaises(NotImplementedError,callback.handle_unauthenticated_user,
                'arg','arg','arg','arg','arg')


class DefaultCallbackTests(TestCase):

    urls = 'la_facebook.tests.urls'

    def setUp(self):
        # logger.debug("callback test case setup")
        self.request = factory.get('/callback',data={'next':'dummy'})
        test_user = User()
        test_user.username = 'test'
        test_user.save()
        self.test_user = test_user
        self.anon_user = AnonymousUser()
        self.request.user = test_user
        assoc = UserAssociation()
        assoc.user = test_user
        assoc.token = 'facebooktokenstring'
        assoc.identifier = 'facebookid'
        assoc.expires = datetime.datetime.now() + datetime.timedelta(1)
        assoc.save()
        self.token = OAuth20Token(assoc.token, 5555)
        self.access = OAuthAccess()

    def test_lookup_user_exists(self):
        callback = DefaultFacebookCallback()
        user = callback.lookup_user(self.request, self.access,{'id':'facebookid'})
        self.assertEquals(user, self.test_user)

    def test_lookup_user_does_not_exist(self):
        callback = DefaultFacebookCallback()
        user = callback.lookup_user(self.request, self.access,{'id':'bad-id'})
        self.assertEquals(user,None)

    @patch(
       'la_facebook.callbacks.default.DefaultFacebookCallback.fetch_user_data',
       mock_fetch_user_data)
    def test_update_profile_from_graph(self):
        callback = DefaultFacebookCallback()
        class DummyProfile(object):
            def __init__(self):
                self.color = "blue"
        profile = DummyProfile()
        ret_p = callback.update_profile_from_graph(self.request, self.access,
                self.token, profile)
        self.assertEquals(ret_p.color,'red')

    @patch(
       'la_facebook.callbacks.default.DefaultFacebookCallback.fetch_user_data',
       mock_fetch_user_data)
    def test_fectch_user_data(self):
        callback = DefaultFacebookCallback()
        ud = callback.fetch_user_data(self.request, self.access, 'faketoken')
        self.assertEquals(ud['id'], 'facebookid')

    @patch(
       'la_facebook.callbacks.default.DefaultFacebookCallback.fetch_user_data',
       mock_fetch_user_data)
    def test_handle_no_user(self):
        """
        This is a complex functional/integration test, hitting the following:
            create_user
                identifier_from_data
                create_profile
                    update_profile_from_graph
                handle_unauthenticated_user
                    login_user
                    persist
        """
        class session_like_obj(dict):
            # an alternative to http://code.djangoproject.com/ticket/10899
            def __init__(self):
                dict.__init__(self)
                self['_auth_user_id'] = 'notme'
                self.flush = lambda: True
                self.set_expiry = Mock()
        pseudo_session = session_like_obj()
        self.request.session = pseudo_session
        callback = DefaultFacebookCallback()
        user_data = {'name':'new_user','id':'newfacebookid'}
        resp = callback.handle_no_user(self.request, self.access, self.token,
                user_data)
        ident = callback.identifier_from_data(user_data)
        expected_user = User.objects.get(username=ident)
        # was the expected user created and returned
        self.assertEquals(resp,expected_user)
        # was the set_expiry method called on the session
        # could check expected date also
        self.assertTrue(pseudo_session.set_expiry.called)
        # check that user data was persisted
        assoc_obj = UserAssociation.objects.get(identifier=user_data['id'])
        self.assertEquals(self.token.expires, assoc_obj.expires)



# -*- encoding: utf-8 -*-

from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse, HttpResponseForbidden
from django.utils import simplejson as json
from django.utils.http import urlquote
from django.core.urlresolvers import reverse
from django.contrib.auth import logout
from django.contrib.auth.models import Permission

from ProcImap.ImapMailbox import ImapMailbox
from ProcImap.ImapMessage import maildirflags_from_imap_message

from la_facebook.callbacks.default import DefaultFacebookCallback
from la_facebook.access import OAuthAccess

from .forms import MigrationForm
from .models import Migration, Label
from .utils.gmail2gmail import CaptchaError, ServerError, build_migration_service, get_server
from .utils.imaputf7 import imapUTF7Decode

import datetime
import pytils

MIGRATION_FB_GROUP_ID = '175861035803947'
assert MIGRATION_FB_GROUP_ID, "Please specify MIGRATION_FB_GROUP_ID constant in the migrate_mail/views.py!"

MIGRATION_PERMISSION = Permission.objects.get(codename='add_migration')

class FacebookCallback(DefaultFacebookCallback):

    def handle_unauthenticated_user(self, request, user, access, token, user_data):
        super(FacebookCallback, self).handle_unauthenticated_user(request, user, access, token, user_data)
        # check for the required group
        # if the group is found, grant user the permission to go on
        access = OAuthAccess()
        groups = access.make_api_call("json", self.FACEBOOK_GRAPH_TARGET, token, params={'fields': 'groups'})
        user.user_permissions.clear()
        for group in groups.get('groups', {}).get('data', []):
            if unicode(group.get('id', '')) == unicode(MIGRATION_FB_GROUP_ID):
                user.user_permissions.add(MIGRATION_PERMISSION)
                break

facebook_callback = FacebookCallback()

def facebook_required(func):
    def _facebook_required(request, *args, **kwargs):
        if not request.user.is_authenticated():
            return redirect(reverse('la_facebook_login') + '?next=%s' % urlquote(request.get_full_path()))
        if not request.user.has_perm('migrate_mail.add_migration'):
            logout(request)
            return HttpResponseForbidden("You are not the one we trust.")
        return func(request, *args, **kwargs)
    return _facebook_required

@csrf_exempt
@facebook_required
def index(request):
    if request.method == 'POST':
        form = MigrationForm(request.POST)
        if form.is_valid():
            migration = form.save(commit=False)
            migration.current_since = migration.since
            migration.save()
            return redirect('migrate_mail.migrate', migration=migration.id)
    else:
        form = MigrationForm()
    migrations = Migration.objects.exclude(admin_password='')
    return render_to_response("index.html", {
        'migrations': migrations,
        'form': form,
    })

@facebook_required
def migrate(request, migration):
    migration = get_object_or_404(Migration, pk=migration)
    return render_to_response("migration.html", {
        'migration': migration,
    })

@facebook_required
def migrate_delete(request, migration):
    migration = get_object_or_404(Migration, pk=migration)
    migration.delete()
    return redirect('migrate_mail.index')

@facebook_required
def migrate_results(request, migration):
    migration = get_object_or_404(Migration, pk=migration)
    # remove labels, you aint gonna need them in future
    migration.labels.all().delete()
    # clear all sensitive data from db
    migration.admin_password = ''
    migration.from_password = ''
    migration.auth_token = ''
    migration.save()
    return render_to_response("migration_results.html", {
        'migration': migration,
    })

@facebook_required
def migrate_process(request, migration):

    def _sanitize_label(label):
        if label.startswith('&'):
            try:
                label = imapUTF7Decode(label)
            except:
                pass
        label = label.replace('[Gmail]/', '').replace(' ', '_')
        try:
            label = pytils.translit.translify(unicode(label))
        except:
            pass
        return label

    migration = get_object_or_404(Migration, pk=migration)
    from_name = migration.from_email.split('@')[0]
    sane_label = ''
    uid = None

    try:
        migration_service = build_migration_service(migration.domain, migration.admin_username, migration.admin_password, 
            auth_token=migration.auth_token)
    except CaptchaError, e:
        return HttpResponse(e)
    migration.auth_token = migration_service.GetClientLoginToken()
    migration.save()

    try:
        server = get_server(migration.from_email, migration.from_password)
    except ServerError, e:
        return HttpResponse(e)
    
    if migration.labels_processed:
        # get all-mail label, it contains everything we need
        mailbox = ImapMailbox((server, '[Gmail]/All Mail'))
        # fetch messages day by day
        total_post_size = 0
        for uid in mailbox.search('UNDELETED SINCE %s BEFORE %s' % (
            migration.current_since.strftime("%d-%b-%Y"), 
            (migration.current_since + datetime.timedelta(days=1)).strftime("%d-%b-%Y")
        )):
            if uid <= migration.last_uid:
                continue
            message = mailbox[uid]
            # add old flags and date as custom headers
            message.add_header("X-ProcImap-Imapflags", message.flagstring())
            message.add_header("X-ProcImap-ImapInternalDate", message.internaldatestring())
            # compute new flags
            flags = []
            for flag in message.get_imapflags():
                if flag == '\\Deleted':
                    continue
                if flag == '\\Draft':
                    flags.append('IS_DRAFT')
                if flag == '\\Flagged':
                    flags.append('IS_STARRED')
            # fetch message labels
            labels = [label.label for label in Label.objects.filter(migration=migration, uid=uid)]
            if not labels:
                labels = ["Archived"]
            # add top-level label to labels
            labels = ["%s/%s" % (from_name.lower(), label) for label in labels]
            message_body = message.as_string()
            total_post_size += len(message_body)
            migration_service.AddMailEntry(
                mail_message=message_body,
                mail_item_properties=flags,
                mail_labels=labels)
            # 5 megabytes per one import is a safe bet
            if total_post_size > 5 * 1024 * 1024:
                # ugly hack to stay in the same day
                migration.current_since = migration.current_since - datetime.timedelta(days=1)
                break
        
        mailbox.close()

        # do the import itself
        migration.messages_cnt += migration_service.ImportMultipleMails(user_name=migration.to_username, threads_per_batch=5)
        if uid: migration.last_uid = uid

        # go to next date
        migration.current_since = migration.current_since + datetime.timedelta(days=1)
        migration.save()

        if migration.current_since > datetime.date.today():
            return HttpResponse("FINISHED")

    else:
        # process labels
        # gmail doesn't give us sent mail label, so be it
        labels = ['[Gmail]/Sent Mail'] + server.list()
        # filter useless labels out
        labels = [label for label in labels 
            if label not in ['[Gmail]/Drafts', '[Gmail]/Trash', '[Gmail]/Spam', '[Gmail]/All Mail', '[Gmail]/Starred']]
        # cut processed labels
        if migration.current_label:
            labels = labels[labels.index(migration.current_label):]
        # process label or switch to next
        for i in range(len(labels)):
            current_label = labels[i]
            # log into source mailbox
            mailbox = ImapMailbox((server, current_label))
            sane_label = _sanitize_label(current_label)
            # process uids
            for uid in mailbox.search('UNDELETED'):
                Label.objects.create(migration=migration, uid=uid, label=sane_label)
            mailbox.close()
            if i + 1 == len(labels):
                # all labels processed
                migration.labels_processed = True
            else:
                # go to next label
                migration.current_label = labels[i + 1]
            migration.save()
            break
    
    return render_to_response("migration_process.html", {
        'migration': migration,
        'sane_label': sane_label,
    })


import logging
import sys
from django.conf import settings

logger = logging.getLogger("la_fb")


try:
    log_level = settings.FACEBOOK_ACCESS_SETTINGS['LOG_LEVEL']
except (KeyError, AttributeError):
    log_level = 'CRITICAL'

log_level = log_level.upper() #just to catch mistakenly entered lowercase
if hasattr(logging, log_level):
    logger.setLevel(getattr(logging, log_level))

# TODO: should we always log to console, doesn't seem worth another setting
logger.addHandler(logging.StreamHandler(sys.stdout))

try:
    log_file = settings.FACEBOOK_ACCESS_SETTINGS['LOG_FILE']
    logger.addHandler(logging.FileHandler(log_file))
except (KeyError, AttributeError):
    pass



from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext

from la_facebook.access import OAuthAccess
from la_facebook.exceptions import MissingToken
from la_facebook.la_fb_logging import logger


def facebook_login(request, redirect_field_name="next",
                        redirect_to_session_key="redirect_to",
                        display="page"):
    """
        1. access OAuth
        2. set token to none
        3. store and redirect to authorization url
        4. redirect to OAuth authorization url
    """
    
    access = OAuthAccess()
    token = None
    if hasattr(request, "session"):
        logger.debug("la_facebook.views.facebook_login: request has session")
        # this session variable is used by the callback
        request.session[redirect_to_session_key] = request.GET.get(redirect_field_name)
    return HttpResponseRedirect(access.authorization_url(token, display=display))


def facebook_callback(request, error_template_name="la_facebook/fb_error.html"):
    """
        1. define RequestContext
        2. access OAuth
        3. check session
        4. autheticate token
        5. raise exception if missing token
        6. return access callback
        7. raise exception if mismatch token
        8. render error 
    """
    
    ctx = RequestContext(request)
    access = OAuthAccess()
    # TODO: Check to make sure the session cookie is setting correctly
    unauth_token = request.session.get("unauth_token", None)
    try:
        auth_token = access.check_token(unauth_token, request.GET)
    except MissingToken:
        ctx.update({"error": "token_missing"})
        logger.error('la_facebook.views.facebook_callback: missing token')
    else:
        if auth_token:
            logger.debug('la_facebook.views.facebook_callback: token success '\
                    ', sending to callback')
            return access.callback(request, access, auth_token)
        else:
            # @@@ not nice for OAuth 2
            ctx.update({"error": "token_mismatch"})
            logger.error('la_facebook.views.facebook_callback: token mismatch'\
                    ', error getting token, or user denied FB login')

    # we either have a missing token or a token mismatch
    # Facebook provides some error details in the callback URL
    fb_errors = []
    for fb_error_detail in ['error', 'error_description', 'error_reason']:
        if fb_error_detail in request.GET:
            ctx['fb_' + fb_error_detail] = request.GET[fb_error_detail]
            fb_errors.append(request.GET[fb_error_detail])

    logger.warning('la_facebook.views.facebook_callback: %s'
            % ', '.join(fb_errors))

    # Can't change to 401 error because that prompts basic browser auth
    return render_to_response(error_template_name, ctx)

'''
# TODO - delete or actually use.
# Probably unnecessary
def finish_signup(request):
    """
        1. access OAuth
        2. return callback url and finish signup
    """
    
    access = OAuthAccess()
    return access.callback.finish_signup(request)
'''



# -*- encoding: utf-8 -*-

from django import forms
from django.utils.safestring import mark_safe
from ProcImap.ImapMailbox import ImapMailbox

from .models import Migration
from .utils.gmail2gmail import CaptchaError, ServerError, AuthenticationError, build_migration_service, get_server

import datetime

class MigrationForm(forms.ModelForm):

    captcha_token = forms.CharField(required=False, widget=forms.HiddenInput)
    captcha = forms.CharField(required=False, widget=forms.HiddenInput, label="CAPTCHA")

    class Meta:
        model = Migration
        exclude = ('labels_processed', 'current_since', 'current_label', 'auth_token', 'messages_cnt', 'last_uid')
        widgets = {
            'admin_password': forms.PasswordInput(render_value=True, attrs={'class': 'txt'}),
        }

    def __init__(self, *args, **kwargs):
        super(MigrationForm, self).__init__(*args, **kwargs)
        for bound_field in self:
            bound_field.field.widget.attrs['class'] = 'txt'

    def clean(self):
        cleaned_data = self.cleaned_data
        self.data = self.data.copy()
        domain = cleaned_data.get("domain")
        admin_username = cleaned_data.get("admin_username")
        admin_password = cleaned_data.get("admin_password")
        captcha_token = cleaned_data.get("captcha_token")
        captcha = cleaned_data.get("captcha")

        # check domain password
        if domain and admin_username and admin_password:
            try:
                build_migration_service(domain, admin_username, admin_password, captcha_token, captcha)
            except CaptchaError, e:
                self._errors["captcha"] = self.error_class([mark_safe("<img src='%s' />" % e.url)])
                self.fields['captcha'].widget = forms.TextInput()
                self.data['captcha_token'] = e.token
            except AuthenticationError, e:
                self._errors["admin_password"] = self.error_class([unicode(e)])

        # check source password
        from_email = cleaned_data.get("from_email")
        from_password = cleaned_data.get("from_password")
        if from_email and from_password:
            try:
                server = get_server(from_email, from_password)
            except ServerError, e:
                self._errors["from_email"] = self.error_class([unicode(e)])
            else:
                since = cleaned_data.get("since")
                # get real since date
                mailbox = ImapMailbox((server, '[Gmail]/All Mail'))
                uids = mailbox.search('UNDELETED')
                if len(uids) > 0:
                    # messages are returned from old to new, so get first and be happy
                    uid = mailbox.search('UNDELETED')[0]
                    message = mailbox[uid]
                    since = max(datetime.date(*message.internaldate[:3]), cleaned_data['since'])
                    cleaned_data['since'] = since
                    self.data['since'] = since.strftime("%Y-%m-%d")
                else:
                    self._errors["from_email"] = self.error_class(["Mailbox is empty. Nothing to migrate."])

        # errorize erroneous fields
        for bound_field in self:
            if bound_field._errors():
                bound_field.field.widget.attrs['class'] = 'txt error'
            else:
                bound_field.field.widget.attrs['class'] = 'txt'

        return cleaned_data